// Copyright (c) 2013, the Dart project authors.  Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

library nested_test;

import 'dart:utf';
import 'package:unittest/unittest.dart';
import 'package:csslib/parser.dart';
import 'package:csslib/visitor.dart';
import 'testing.dart';

List optionsCss = ['--no-colors', 'memory'];

void simpleNest() {
  final errors = [];
  final input = '''
div span { color: green; }
#header {
  color: red;
  h1 {
    font-size: 26px;
    font-weight: bold;
  }
  p {
    font-size: 12px;
    a {
      text-decoration: none;
    }
  }
  background-color: blue;
}
div > span[attr="foo"] { color: yellow; }
''';

  final generated =
r'''div span {
  color: #008000;
}
#header {
  color: #f00;
  background-color: #00f;
}
#header p {
  font-size: 12px;
}
#header p a {
  text-decoration: none;
}
#header h1 {
  font-size: 26px;
  font-weight: bold;
}
div > span[attr="foo"] {
  color: #ff0;
}''';

  var stylesheet = compileCss(input, errors: errors, opts: optionsCss);

  expect(stylesheet != null, true);
  expect(errors.isEmpty, true, reason: errors.toString());
  expect(prettyPrint(stylesheet), generated);
}

void complexNest() {
  final errors = [];
  final input = '''
@font-face  { font-family: arial; }
div { color: #f0f0f0; }
#header + div {
  color: url(abc.png);
  *[attr="bar"] {
    font-size: 26px;
    font-weight: bold;
  }
  p~ul {
    font-size: 12px;
    :not(p) {
      text-decoration: none;
      div > span[attr="foo"] { color: yellow; }
    }
  }
  background-color: blue;
  span {
    color: red;
    .one { color: blue; }
    .two { color: green; }
    .three { color: yellow; }
    .four {
       .four-1 { background-color: #00000f; }
       .four-2 { background-color: #0000ff; }
       .four-3 { background-color: #000fff; }
       .four-4 {
         height: 44px;
         .four-4-1 { height: 10px; }
         .four-4-2 { height: 20px; }
         .four-4-3 { height: 30px; }
         width: 44px;
       }
    }
  }
}
span { color: #1f1f2f; }
''';

  final generated = r'''@font-face  {
  font-family: arial;
}
div {
  color: #f0f0f0;
}
#header + div {
  color: url("abc.png");
  background-color: #00f;
}
#header + div span {
  color: #f00;
}
#header + div span .four {
}
#header + div span .four .four-4 {
  height: 44px;
  width: 44px;
}
#header + div span .four .four-4 .four-4-3 {
  height: 30px;
}
#header + div span .four .four-4 .four-4-2 {
  height: 20px;
}
#header + div span .four .four-4 .four-4-1 {
  height: 10px;
}
#header + div span .four .four-3 {
  background-color: #000fff;
}
#header + div span .four .four-2 {
  background-color: #00f;
}
#header + div span .four .four-1 {
  background-color: #00000f;
}
#header + div span .three {
  color: #ff0;
}
#header + div span .two {
  color: #008000;
}
#header + div span .one {
  color: #00f;
}
#header + div p ~ ul {
  font-size: 12px;
}
#header + div p ~ ul :not(p) {
  text-decoration: none;
}
#header + div p ~ ul :not(p) div > span[attr="foo"] {
  color: #ff0;
}
#header + div *[attr="bar"] {
  font-size: 26px;
  font-weight: bold;
}
span {
  color: #1f1f2f;
}''';

  var stylesheet = compileCss(input, errors: errors, opts: optionsCss);

  expect(stylesheet != null, true);
  expect(errors.isEmpty, true, reason: errors.toString());
  expect(prettyPrint(stylesheet), generated);
}

void simpleThis() {
  final errors = [];
  final input = '''#header {
  h1 {
    font-size: 26px;
    font-weight: bold;
  }
  p { font-size: 12px;
    a { text-decoration: none;
      &:hover { border-width: 1px }
    }
  }
}
''';

  final generated = r'''#header h1 {
  font-size: 26px;
  font-weight: bold;
}
#header  p {
  font-size: 12px;
}
#header p a {
  text-decoration: none;
}
#header p a:hover {
border-width: 1px;
}''';

  var stylesheet = compileCss(input, errors: errors, opts: optionsCss);

  expect(stylesheet != null, true);
  expect(errors.isEmpty, true, reason: errors.toString());
  expect(prettyPrint(stylesheet), generated);
}

main() {
  test('Simple nesting', simpleNest);
  test('Complex nesting', complexNest);
//  test('Simple Nest w/ &', simpleThis);
}
